module EasyAdminUi
  module EasyAdminUiHelper
    def table(items, columns, actions, options = {}, &block)
      options ||= {}
      html = ''.html_safe
      html << table_head(columns, actions, options)
      html << render(:collection => items, :partial => options[:partial])
      html << my_table_end
      html
    end

    def table_head(columns, actions, options = {})
      options[:class] ||= 'easy'
      html = ''.html_safe
      html << "<table class='#{options[:class]}' id='#{options[:id]}'>".html_safe
      html <<
        content_tag(:thead,
          content_tag(:tr, (
              columns.map do |title, th_class|
                th_class ||= []
                content_tag(:th,
                  [
                    if th_class.include?('sortable')
                      content_tag(:div,
                        link_to('', '#', :class => 'sort_up') +
                        link_to('', '#', :class => 'sort_down'),
                        {:class => 'sort'})
                    else
                      nil
                    end,
                    title
                  ].compact.join('').html_safe,
                  :class => th_class)
              end
            ).join('').html_safe + (actions ? content_tag(:th, '', :class => 'actions') : '')
          )
        )
      html << options[:body_id] ? "<tbody id=\"#{options[:body_id]}\">" : '<tbody>'
      html
    end

    def table_row(options = {})
      options[:actions] ||= []

      content_tag(:tr,
        [
          # Could pass:
          #
          # [category.created_at, {:class => '', :onmouseover => "javascript: blah"}]
          # [category.created_at, 'my_class_name']
          #
          options[:cells].map do |txt, cell_opts|
            cell_opts = {:class => cell_opts} if cell_opts.is_a?(String)
            cell_opts ||= {}
            content_tag(:td, txt, cell_opts)
          end,
          options[:actions].blank? ? '' : content_tag(:td, options[:actions].join(' ').html_safe, :class => "act")
        ].join(' ').html_safe,
        :id => options[:id], :class => ["#{cyc = cycle("even", "odd")}", options[:class]].join(' ')).html_safe
    end

    def my_table_end
      '</tbody></table>'.html_safe
    end

    def right_align(content)
      [content, 'aright']
    end

    def admin_show_link(item, url_options = {}, html_options = {})
      link_to(show_icon_image_tag, eval("#{items_to_url(item)}(*item, url_options)"), html_options)
    end

    def admin_new_link(url_options = {}, html_options = {})
      content_tag(:span, raw(' - ') +
        link_to('New', {action: 'new'}.merge(url_options), html_options))
    end

    def admin_edit_link(item, url_options = {}, html_options = {})
      link_to(edit_icon_image_tag, eval("edit_#{items_to_url(item)}(*item, url_options)"), html_options)
    end

    # Use if below /admin/.
    def admin_delete_link_with_confirmation(item, url_options = {}, options = {})
      link_to(delete_icon_image_tag, eval("#{items_to_url(item)}(*item, url_options)"), class: "admin_delete", 'data-id' => item.to_param.gsub('/', '_'), 'data-hide-id' => options[:hide_id])
    end

    def items_to_url(item)
      "admin_" +
      if item.is_a?(Array)
        item.map { |obj|
          if obj.new_record?
            obj.class.to_s.pluralize.underscore
          else
            obj.class.to_s.singularize.underscore
          end
        }.join('_')
      else
        item.class.to_s.underscore.singularize.to_s
      end +
      "_path"
    end

    def partial_exists?(filename)
      ActionController::Base.view_paths.any? {|path| File.exists?(File.join(path, self.controller_path, filename)) }
    end

    def filename_from_partial(partial)
      "_#{partial}.html.erb"
    end

    # Render partial if it exists.
    def render_optional(partial)
      return '' unless partial_exists?(filename_from_partial(partial))
      render partial: partial
    end

    # Render the first partial it finds.
    def render_optional_either(*partials)
      ''.tap do |html|
        partials.each do |partial|
          if partial_exists?(filename_from_partial(partial))
            html << render(partial: partial)
            break
          end
        end
      end.html_safe
    end

    def show_icon_image_tag
      image_tag("easy_admin_ui/show.png")
    end

    def edit_icon_image_tag
      image_tag("easy_admin_ui/pencil.png")
    end

    def delete_icon_image_tag
      image_tag('easy_admin_ui/delete.png')
    end
  end
end
